using System;
using System.Collections;
using System.Xml.Serialization;

namespace DeleteDataGridRowIssue
{
	/// <summary>
	/// Summary description for Criteria.
	/// </summary>
	public class Criteria : BaseItem
	{
		
		string _selectedOperator;
		string _value;
		string _dbtype;
		public Criteria() : base()
		{
		}
		public Criteria(string name, string display) : this()
		{
			base.Name = name;
			
		}
		public Criteria(string name, string display, string dbtype) : this(name, display)
		{
			DBType = dbtype;
		}
		public Criteria(string name, string display, string dbtype, string selectedOperator, string val) : this(name, display, dbtype)
		{
			SelectedOperator = selectedOperator;
			Value = val;
		}
		[XmlAttribute("operator")]
		public string SelectedOperator
		{
			get { return _selectedOperator; }
			set { _selectedOperator = value; }
		}
		[XmlAttribute("value")]
		public string Value
		{
			get { return _value; }
			set { _value = value; }
		}
		[XmlAttribute("dbtype")]
		public string DBType
		{
			get { return _dbtype; }
			set { _dbtype = value; }
		}
	}

	public class CriteriaList : BaseList
	{
		public CriteriaList()
		{
		}
		public int Add(Criteria c)
		{
			return List.Add(c);
		}
		public Criteria this[int index]
		{
			get { return InnerList[index] as Criteria; }
		}
		public string[] GetNames()
		{
			string[] names = new string[InnerList.Count];
			for(int index=0; index<InnerList.Count; index++)
			{
				Criteria c = this[index];
				names[index] = c.Name;
			}
			return names;
		}
		public string[] GetOperators()
		{
			string[] operators = new string[InnerList.Count];
			for(int index=0; index<InnerList.Count; index++)
			{
				Criteria c = this[index];
				operators[index] = c.SelectedOperator;
			}
			return operators;
		}
		public string[] GetValues()
		{
			string[] values = new string[InnerList.Count];
			for(int index=0; index<InnerList.Count; index++)
			{
				Criteria c = this[index];
				values[index] = c.Value;
			}
			return values;
		}
		public string[] GetTypes()
		{
			string[] types = new string[InnerList.Count];
			for(int index=0; index<InnerList.Count; index++)
			{
				Criteria c = this[index];
				types[index] = c.DBType;
			}
			return types;
		}

		public int IndexOfCriteriaId(int criteriaId)
		{
			for(int index=0; index < InnerList.Count; index++)
			{
				Criteria c = InnerList[index] as Criteria;
				if (c.Id == criteriaId)
					return index;
			}
			return -1;
		}

		public int IndexOfCriteriaName(string name)
		{
			for(int index=0; index < InnerList.Count; index++)
			{
				Criteria c = InnerList[index] as Criteria;
				if (String.Compare(c.Name,name,true)==0)
					return index;
			}
			return -1;
		}
	}

	public class Criterias : BaseItem
	{
		CriteriaList _list = new CriteriaList();
		public Criterias()
		{
		}
		[XmlElement("Criteria", typeof(Criteria))]
		public CriteriaList CriteriaList
		{
			get { return _list; }
			set { _list = value; }
		}
	}

	public abstract class BaseItem : IComparable, ICloneable
	{
		int _id;
		string _name;
		//		DateTime _asOfDateTime = DateTime.Now;
		//		DateTime _updateTS ;
		//		DateTime _createTS;
		//		string _deleteFlag = "N";
		//		StatusType	_status ;
		public BaseItem()
		{
		}
		public BaseItem(int id, string name) : this()
		{
			_id = id;
			_name = name;
		}
		// do not overide Equals.  this gives circular reference error while serializing Notes.
		//		public override bool Equals(object obj)
		//		{
		//			BaseItem item = (BaseItem)obj;
		//			if (string.Compare(this.Name, item.Name, true) == 0)
		//				return true;
		//			return false;
		//		}
		//		public override int GetHashCode()
		//		{
		//			return this.Id;
		//		}
		[XmlAttribute("id")]
		public virtual int Id 
		{
			get { return _id; }
			set { _id = value; }
		}
		[XmlAttribute("name")]
		public virtual string Name 
		{
			get { return _name; }
			set { _name = value; }
		}
		//		[XmlAttribute("asOfDate", DataType="date" )]
		//		public DateTime AsOfDateTime
		//		{
		//			get { return _asOfDateTime; }
		//			set { _asOfDateTime = value; }
		//		}

		//		public DateTime CreateTS
		//		{
		//			get{return _createTS;}
		//			set{_createTS = value;}
		//		}
		//
		//		public DateTime UpdateTS
		//		{
		//			get{return _updateTS;}
		//			set{_updateTS = value;}
		//		}
		//
		//		public string DeleteFlag
		//		{
		//			get{return _deleteFlag;}
		//			set{_deleteFlag = value;}
		//		}

		//		public StatusType Status
		//		{
		//			get
		//			{
		//				if(DeleteFlag == "Y")
		//					_status = StatusType.Delete;
		//				else if(DateTime.Compare(CreateTS,UpdateTS) == 0)
		//					_status  = StatusType.Insert;
		//				else 
		//					_status = StatusType.Update;
		//				
		//				return _status;
		//			}
		//		}

		#region IComparable Members
		public int CompareTo(object obj)
		{
			BaseItem item = obj as BaseItem;
			if(item != null)
				return string.Compare(item.Name, _name);
			else
				return 0;
		}
		#endregion
		#region ICloneable Members
		public object Clone()
		{
			return this.MemberwiseClone();
		}
		#endregion
	
		#region Events (duplicated in BaseItem & BaseList)
		
		#region Event Declaration
		public event SnTypesMessageHandler SnTypesMessage;
		public event SnTypesErrorHandler SnTypesError;
		#endregion

		#region Event Notifications
		protected void DoSnTypesMessage(string message)
		{
			if(SnTypesMessage != null)
				SnTypesMessage(this, message);
		}
		protected void DoSnTypesError(Exception e)
		{
			if(SnTypesError != null)
				SnTypesError(this, e);
		}
		protected void DoSnTypesError(string message)
		{
			if(SnTypesError != null)
				SnTypesError(this, new Exception(message));
		}
		#endregion
		
		#region Error Targets
		public void OnSnTypesMessage(object sender, string message)
		{
			DoSnTypesMessage(message);
		}
		public void OnSnTypesError(object sender, Exception e)
		{
			DoSnTypesError(e);
		}
		#endregion

		#endregion
	}

	public delegate void SnTypesMessageHandler(object sender, string message);
	public delegate void SnTypesErrorHandler(object sender, Exception e);

	public abstract class BaseList : CollectionBase, IBaseList
	{
		public BaseList()
		{
		}
		protected override void OnValidate(object value)
		{	
			if(!(value is BaseItem))
				throw new InvalidCastException();
			base.OnValidate (value);
		}
		//		protected int Add(BaseItem item)
		//		{
		//			return List.Add(item);
		//		}
		//		protected void Remove(BaseItem item)
		//		{
		//			List.Remove(item);
		//		}
		private BaseItem this[int index]
		{
			get { return InnerList[index] as BaseItem; }
		}
		protected virtual BaseItem Last()
		{
			if(Count > 0)
				return InnerList[Count - 1] as BaseItem;
			else
				return null;
		}
		protected virtual BaseItem First()
		{
			if(Count > 0)
				return InnerList[0] as BaseItem;
			else
				return null;
		}
		protected virtual BaseItem FindById(int id)
		{
			for(int index=0; index<InnerList.Count; index++)
			{
				BaseItem item = InnerList[index] as BaseItem;
				if (item != null && item.Id == id)
					return item;
			}
			return null;
		}
		protected virtual BaseItem FindByName(string name)
		{
			for(int index=0; index<InnerList.Count; index++)
			{
				BaseItem item = InnerList[index] as BaseItem;
				if (item != null && (string.Compare(item.Name, name, true) == 0))
					return item;
			}
			return null;
		}
		protected bool Contains(BaseItem item)
		{
			return InnerList.Contains(item);
		}
		protected int IndexOf(BaseItem item)
		{
			return InnerList.IndexOf(item);
		}
		public void CopyTo(Array array, int index)
		{
			InnerList.CopyTo(array, index);
		}
		public bool IsFixedSize
		{
			get { return false; }
		}
		public bool IsReadOnly
		{
			get { return false; }
		}
		public void Sort()
		{
			InnerList.Sort();
		}
		public void Sort(IComparer comparer)
		{
			InnerList.Sort(comparer);
		}
		public Hashtable CreateIdIndex()
		{
			Hashtable index = new Hashtable(this.Count);
			for(int i = 0; i < this.Count; i++)
			{
				BaseItem obj = this[i];
				if(obj.Id <= 0)
					throw new InvalidOperationException("Invalid object Id");
				index.Add(obj.Id, obj);
			}
			return index;
		}
		public Hashtable CreateNameIndex(bool toUpper)
		{
			Hashtable index = new Hashtable(this.Count);
			for(int i = 0; i < this.Count; i++)
			{
				BaseItem obj = this[i];
				if(obj.Name == null || obj.Name.Length == 0)
					throw new InvalidOperationException("Invalid object name");
				if(toUpper)
					index.Add(obj.Name.ToUpper(), obj);
				else
					index.Add(obj.Name, obj);
			}
			return index;
		}

		#region Events (duplicated in BaseItem & BaseList)
		
		#region Event Declaration
		public event SnTypesMessageHandler SnTypesMessage;
		public event SnTypesErrorHandler SnTypesError;
		#endregion

		#region Event Notifications
		protected void DoSnTypesMessage(string message)
		{
			if(SnTypesMessage != null)
				SnTypesMessage(this, message);
		}
		protected void DoSnTypesError(Exception e)
		{
			if(SnTypesError != null)
				SnTypesError(this, e);
		}
		protected void DoSnTypesError(string message)
		{
			if(SnTypesError != null)
				SnTypesError(this, new Exception(message));
		}
		#endregion
		
		#region Error Targets
		public void OnSnTypesMessage(object sender, string message)
		{
			DoSnTypesMessage(message);
		}
		public void OnSnTypesError(object sender, Exception e)
		{
			DoSnTypesError(e);
		}
		#endregion

		#endregion
	}
	public interface IBaseItem
	{
		int Id { get; set; }
		string Name { get; set; }
		//DateTime AsOfDateTime { get; set; }
	}

	public interface IBaseList : IList, ICollection, IEnumerable
	{
	}
}
